@@ -1,42 +1,31 @@
 # -*- coding: utf-8 -*-
-from cms.exceptions import AppAllreadyRegistered
+from cms.exceptions import AppAlreadyRegistered
+from cms.utils.django_load import load, iterload_objects
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.utils.importlib import import_module
+import warnings
 
 class ApphookPool(object):
     def __init__(self):
         self.apps = {}
         self.discovered = False
         self.block_register = False
-        
+
     def discover_apps(self):
         if self.discovered:
             return
         #import all the modules
         if settings.CMS_APPHOOKS:
-            
-            for app in settings.CMS_APPHOOKS:
-                self.block_register = True
-                path = ".".join(app.split(".")[:-1])
-                class_name = app.split(".")[-1]
-                module = import_module(path)
-                cls = getattr(module, class_name, None)
-                if cls is None:
-                    raise ImproperlyConfigured(
-                        "Cannot find class %s" % app
-                    )
+            self.block_register = True
+            for cls in iterload_objects(settings.CMS_APPHOOKS):
                 self.block_register = False
                 self.register(cls)
+                self.block_register = True
+            self.block_register = False
         else:
-            for app in settings.INSTALLED_APPS:
-                cms_app = '%s.cms_app' % app
-                try:
-                    import_module(cms_app)
-                except ImportError:
-                    pass
+            load('cms_app')
         self.discovered = True
-        
+
     def clear(self):
         self.apps = {}
         self.discovered = False
@@ -45,11 +34,18 @@ class ApphookPool(object):
         if self.block_register:
             return
         from cms.app_base import CMSApp
-        assert issubclass(app, CMSApp)
-        if app.__name__ in self.apps.keys():
-            raise AppAllreadyRegistered, "[%s] an cms app with this name is already registered" % app.__name__
-        self.apps[app.__name__] = app
-        
+        # validate the app
+        if not issubclass(app, CMSApp):
+            raise ImproperlyConfigured('CMS Apps must inherit '
+                                       'cms.app_base.CMSApp, %r does not' % app)
+        if hasattr(app, 'menu') and not app.menus:
+            warnings.warn("You define a 'menu' attribute on your CMS App %r, "
+                "but the 'menus' attribute is empty, did you make a typo?")
+        name = app.__name__
+        if name in self.apps.keys():
+            raise AppAlreadyRegistered, "[%s] an cms app with this name is already registered" % name
+        self.apps[name] = app
+
     def get_apphooks(self):
         self.discover_apps()
         hooks = []
@@ -59,8 +55,9 @@ class ApphookPool(object):
         # Unfortunately, we loose the ordering since we now have a list of tuples. Let's reorder by app_name:
         hooks = sorted(hooks, key=lambda hook: hook[1])
         return hooks
-    
+
     def get_apphook(self, app_name):
+        self.discover_apps()
         try:
             return self.apps[app_name]
         except KeyError:
