@@ -15,18 +15,18 @@ class LinkPlugin(CMSPluginBase):
     
     def render(self, context, instance, placeholder):
         if instance.mailto:
-            link = u"mailto:%s" % settings.dbgettext(instance.mailto)
+            link = u"mailto:%s" % instance.mailto
         elif instance.url:
-            link = settings.dbgettext(instance.url)
+            link = instance.url
         elif instance.page_link:
             link = instance.page_link.get_absolute_url()
         else:
             link = ""
         context.update({
-            'name':settings.dbgettext(instance.name),
-            'link':link, 
-            'placeholder':placeholder,
-            'object':instance
+            'name': instance.name,
+            'link': link, 
+            'placeholder': placeholder,
+            'object': instance
         })
         return context
     
@@ -62,6 +62,6 @@ class LinkPlugin(CMSPluginBase):
         return FakeForm(Form, site)
         
     def icon_src(self, instance):
-        return settings.CMS_MEDIA_URL + u"images/plugins/link.png"
+        return settings.STATIC_URL + u"cms/images/plugins/link.png"
     
 plugin_pool.register_plugin(LinkPlugin)
