@@ -1,11 +1,16 @@
 # -*- coding: utf-8 -*-
 # TODO: this is just stuff from utils.py - should be splitted / moved
 from cms.utils.i18n import get_default_language
+from distutils.version import LooseVersion
 from django.conf import settings
+from django.core.files.storage import get_storage_class
 from django.core.urlresolvers import reverse
 from django.http import HttpResponse, HttpResponseRedirect
 from django.shortcuts import render_to_response
 from django.template import RequestContext
+from django.utils.functional import LazyObject
+import django
+import os
 import urllib
 
 
@@ -39,8 +44,6 @@ def get_language_from_request(request, current_page=None):
     """
     Return the most obvious language according the request
     """
-    if settings.CMS_DBGETTEXT: 
-        return get_default_language()
     language = request.REQUEST.get('language', None)
     if language:
         if not language in dict(settings.CMS_LANGUAGES).keys():
@@ -74,4 +77,28 @@ def get_page_from_request(request):
     warn("'cms.utils.get_page_from_request' is deprecated in favor of "
          "'cms.utils.page_resolver.get_page_from_request' and will be removed "
          "in Django-CMS 2.2.", DeprecationWarning)
-    return new(request)
\ No newline at end of file
+    return new(request)
+
+
+"""
+The following class is taken from https://github.com/jezdez/django/compare/feature/staticfiles-templatetag
+and should be removed and replaced by the django-core version in 1.4
+"""
+default_storage = 'django.contrib.staticfiles.storage.StaticFilesStorage'
+if LooseVersion(django.get_version()) < LooseVersion('1.3'):
+    default_storage = 'staticfiles.storage.StaticFilesStorage'
+
+
+class ConfiguredStorage(LazyObject):
+    def _setup(self):
+        self._wrapped = get_storage_class(getattr(settings, 'STATICFILES_STORAGE', default_storage))()
+
+configured_storage = ConfiguredStorage()
+
+def cms_static_url(path):
+    '''
+    Helper that prefixes a URL with STATIC_URL and cms
+    '''
+    if not path:
+        ''
+    return configured_storage.url(os.path.join('cms', path))
