@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
+from cms.exceptions import NoPermissionsException
+from cms.models import Page, PagePermission, GlobalPagePermission
+from cms.plugin_pool import plugin_pool
+from django.conf import settings
 from django.contrib.auth.models import User, Group
+from django.contrib.sites.models import Site
 from django.db.models import Q
 from django.utils.translation import ugettext_lazy as _
-from cms.models import Page, PagePermission, GlobalPagePermission
-from cms.exceptions import NoPermissionsException
-from django.contrib.sites.models import Site
-
 
 
 
@@ -18,20 +19,21 @@ except ImportError:
 _thread_locals = local()
 
 def set_current_user(user):
-    """Assigns current user from request to thread_locals, used by
+    """
+    Assigns current user from request to thread_locals, used by
     CurrentUserMiddleware.
     """
     _thread_locals.user=user
-
     
 def get_current_user():
-    """Returns current user, or None
+    """
+    Returns current user, or None
     """
     return getattr(_thread_locals, 'user', None)
 
-
 def has_page_add_permission(request):
-    """Return true if the current user has permission to add a new page. This is
+    """
+    Return true if the current user has permission to add a new page. This is
     just used for general add buttons - only superuser, or user with can_add in
     globalpagepermission can add page.
     
@@ -39,7 +41,7 @@ def has_page_add_permission(request):
     Special case occur when page is going to be added from add page button in
     change list - then we have target and position there, so check if user can
     add page under target page will occur. 
-    """        
+    """
     opts = Page._meta
     if request.user.is_superuser:
         return True
@@ -51,7 +53,7 @@ def has_page_add_permission(request):
     if target is not None:
         try:
             page = Page.objects.get(pk=target)
-        except:
+        except Page.DoesNotExist:
             return False
         if (request.user.has_perm(opts.app_label + '.' + opts.get_add_permission()) and
             GlobalPagePermission.objects.with_user(request.user).filter(can_add=True, sites__in=[page.site_id])):
@@ -71,29 +73,41 @@ def has_page_add_permission(request):
     return False
 
 def has_any_page_change_permissions(request):
-    from cms.utils.plugins import current_site  
-    permissions = PagePermission.objects.filter(page__site=current_site(request)).filter(Q(user=request.user)|Q(group__in=request.user.groups.all()))
-    if permissions.count()>0:
-        return True
-    return False
- 
+    from cms.utils.plugins import current_site
+    return PagePermission.objects.filter(
+            page__site=current_site(request)
+        ).filter((
+            Q(user=request.user) |
+            Q(group__in=request.user.groups.all())
+        )).exists()
+
 def has_page_change_permission(request):
-    """Return true if the current user has permission to change any page. This is
+    """
+    Return true if the current user has permission to change any page. This is
     just used for building the tree - only superuser, or user with can_change in
     globalpagepermission can change a page.
-    """    
-    from cms.utils.plugins import current_site    
+    """
+    from cms.utils.plugins import current_site
     opts = Page._meta
-    if request.user.is_superuser or \
-        (request.user.has_perm(opts.app_label + '.' + opts.get_change_permission()) and
-            (GlobalPagePermission.objects.with_user(request.user).filter(can_change=True, sites__in=[current_site(request)]).count()>0) or 
-            has_any_page_change_permissions(request)):
+    if request.user.is_superuser or (
+        request.user.has_perm(opts.app_label + '.' + opts.get_change_permission()) and (
+            GlobalPagePermission.objects.with_user(request.user).filter(
+                can_change=True, sites__in=[current_site(request)]
+            ).exists()) or has_any_page_change_permissions(request)):
         return True
     return False
 
+def get_any_page_view_permissions(request, page):
+    from cms.utils.plugins import current_site
+    return PagePermission.objects.filter(
+            page__pk=page.pk,
+            page__site=current_site(request),
+            can_view=True)
+
 
 def get_user_permission_level(user):
-    """Returns highest user level from the page/permission hierarchy on which
+    """
+    Returns highest user level from the page/permission hierarchy on which
     user haves can_change_permission. Also takes look into user groups. Higher 
     level equals to lover number. Users on top of hierarchy have level 0. Level 
     is the same like page.level attribute.
@@ -110,8 +124,8 @@ def get_user_permission_level(user):
         2.
     
     """
-    if user.is_superuser or \
-        GlobalPagePermission.objects.with_can_change_permissions(user).count():
+    if (user.is_superuser or
+            GlobalPagePermission.objects.with_can_change_permissions(user).exists()):
         # those
         return 0
     try:
@@ -122,7 +136,8 @@ def get_user_permission_level(user):
     return permission.page.level
 
 def get_subordinate_users(user):
-    """Returns users queryset, containing all subordinate users to given user 
+    """
+    Returns users queryset, containing all subordinate users to given user
     including users created by given user and not assigned to any page.
     
     Not assigned users must be returned, because they shouldn't get lost, and
@@ -156,13 +171,22 @@ def get_subordinate_users(user):
     if user.is_superuser or \
             GlobalPagePermission.objects.with_can_change_permissions(user):
         return User.objects.all() 
-    
-    page_id_allow_list = Page.permissions.get_change_permissions_id_list(user)
-    
-    user_level = get_user_permission_level(user)
-    
+    site = Site.objects.get_current()
+    page_id_allow_list = Page.permissions.get_change_permissions_id_list(user, site)
+    try:
+        user_level = get_user_permission_level(user)
+    except NoPermissionsException:
+        # no permission so only staff and no page permissions 
+        qs = User.objects.distinct().filter(
+                Q(is_staff=True) & 
+                Q(pageuser__created_by=user) & 
+                Q(pagepermission__page=None)
+        )
+        qs = qs.exclude(pk=user.id).exclude(groups__user__pk=user.id)
+        return qs
+    # normal query
     qs = User.objects.distinct().filter(
-        Q(is_staff=True) &
+        Q(is_staff=True) & 
         (Q(pagepermission__page__id__in=page_id_allow_list) & Q(pagepermission__page__level__gte=user_level)) 
         | (Q(pageuser__created_by=user) & Q(pagepermission__page=None))
     )
@@ -170,15 +194,25 @@ def get_subordinate_users(user):
     return qs
 
 def get_subordinate_groups(user):
-    """Simillar to get_subordinate_users, but returns queryset of Groups instead
+    """
+    Simillar to get_subordinate_users, but returns queryset of Groups instead
     of Users.
     """
-    if user.is_superuser or \
-            GlobalPagePermission.objects.with_can_change_permissions(user):
+    if (user.is_superuser or
+            GlobalPagePermission.objects.with_can_change_permissions(user)):
         return Group.objects.all()
-    
-    page_id_allow_list = Page.permissions.get_change_permissions_id_list(user)
-    user_level = get_user_permission_level(user)
+    site = Site.objects.get_current()
+    page_id_allow_list = Page.permissions.get_change_permissions_id_list(user, site)
+    try:
+        user_level = get_user_permission_level(user)
+    except NoPermissionsException:
+        # no permission no records
+        # page_id_allow_list is empty
+        qs = Group.objects.distinct().filter(
+         Q(pageusergroup__created_by=user) & 
+         Q(pagepermission__page=None)
+        )
+        return qs
     
     qs = Group.objects.distinct().filter(
          (Q(pagepermission__page__id__in=page_id_allow_list) & Q(pagepermission__page__level__gte=user_level)) 
@@ -188,35 +222,15 @@ def get_subordinate_groups(user):
 
 def has_global_change_permissions_permission(user):
     opts = GlobalPagePermission._meta
-    if user.is_superuser or \
-        (user.has_perm(opts.app_label + '.' + opts.get_change_permission()) and
+    if user.is_superuser or (
+        user.has_perm(opts.app_label + '.' + opts.get_change_permission()) and
             GlobalPagePermission.objects.with_user(user).filter(can_change=True)):
         return True
     return False
 
-
-
-def mail_page_user_change(user, created=False, password=""):
-    """Send email notification to given user. Used it PageUser profile creation/
-    update.
-    """
-    from cms.utils.mail import send_mail
-    
-    if created:
-        subject = _('CMS - your user account was created.')
-    else:
-        subject = _('CMS - your user account was changed.')
-    
-    context = {
-        'user': user,
-        'password': password or "*" * 8,
-        'created': created,
-    }
-    send_mail(subject, 'admin/cms/mail/page_user_change.txt', [user.email], context, 'admin/cms/mail/page_user_change.html')
-
-
 def has_generic_permission(page_id, user, attr, site):
-    """Permission getter for single page with given id.
+    """
+    Permission getter for single page with given id.
     """
     func = getattr(Page.permissions, "get_%s_id_list" % attr)
     permission = func(user, site)
@@ -224,7 +238,8 @@ def has_generic_permission(page_id, user, attr, site):
 
 
 def get_user_sites_queryset(user):
-    """Returns queryset of all sites available for given user.
+    """
+    Returns queryset of all sites available for given user.
     
     1.  For superuser always returns all sites.
     2.  For global user returns all sites he haves in global page permissions 
@@ -236,21 +251,33 @@ def get_user_sites_queryset(user):
     if user.is_superuser:
         return qs
     
-    global_ids = GlobalPagePermission.objects.with_user(user) \
-        .filter(Q(can_add=True) | Q(can_change=True)).values_list('id', flat=True)
+    global_ids = GlobalPagePermission.objects.with_user(user).filter(
+        Q(can_add=True) | Q(can_change=True)
+    ).values_list('id', flat=True)
     
     q = Q()
     if global_ids:
         q = Q(globalpagepermission__id__in=global_ids)
         # haves some global permissions assigned
-        if not qs.filter(q).count():
-            # haves global permissions, but none of sites is specified, so he haves 
-            # access to all sites 
+        if not qs.filter(q).exists():
+            # haves global permissions, but none of sites is specified,
+            # so he haves access to all sites
             return qs
     
-    # add some pages if he haves permission to add / change her    
+    # add some pages if he haves permission to add / change them
     q |= Q(Q(page__pagepermission__user=user) | Q(page__pagepermission__group__user=user)) & \
         (Q(Q(page__pagepermission__can_add=True) | Q(page__pagepermission__can_change=True)))
     return qs.filter(q).distinct()
-    
-    
\ No newline at end of file
+
+
+def has_plugin_permission(user, plugin_type, permission_type):
+    """
+    Checks that a user has permissions for the plugin-type given to performe 
+    the action defined in permission_type
+    permission_type should be 'add', 'change' or 'delete'.
+    """
+    plugin_class = plugin_pool.get_plugin(plugin_type)
+    plugin_model = plugin_class.model
+    plugin_opts = plugin_model._meta
+    return user.has_perm('%s.%s_%s' % (plugin_opts.app_label, permission_type,
+                                      plugin_opts.object_name.lower()))
